using Jypeli;
using Jypeli.Assets;
using Jypeli.Controls;
using Jypeli.Widgets;
using System;

/// @author pepihlaj
/// @version 15.11.2022
/// <summary>
/// HIKO peli
/// </summary>
namespace Hiko
{
    /// <summary>
    /// Tehdään vihuluokka helpottamaan hitpointtien laskussa
    /// </summary>
    class Vihu : PlatformCharacter
    {
        private IntMeter elamalaskuri = new IntMeter(3, 0, 3);
        public IntMeter Elamalaskuri { get { return elamalaskuri; } }
        public Vihu(double leveys, double korkeus)
            :base(leveys, korkeus)
        {
            elamalaskuri.LowerLimit += delegate { this.Destroy(); }; //kun vihun hp tippuu nolliin se poistetaan
        }
    }



    /// <summary>
    /// Peli
    /// </summary>
    public class Hiko : PhysicsGame
    {
        PhysicsObject pelaaja; //varsinainen kontrolloitava pelaaja
        GameObject tahtain; //tähtäin jota käyttäjä voi kontrolloida
        Image taustakuva = LoadImage("Testitaustakuva"); //taustakuva peliin
        IntMeter pistelaskuri; //piste laskuri jota käytetään käyttäjän tehokkuuden laskemiseen
        Label aikanaytto; //tällä saadaan aika käytettävään muotoon
        int jaljellaOlevatVihut; //vihujen lkm kentässä, pelaaja voittaa pelin kun niitä on nolla
        private ScoreList topLista = new ScoreList(10, false, 0); 
        bool tarkistin = true;


        /// <summary>
        /// Tällä taulukolla luodaan peliin kenttä
        /// </summary>
        private static readonly String[] lines =
        {
                  "    X X                 ",
                  "   ______        X      ",
                  "                ____    ",
                  "                        ",
                  "                        ",
                  "    X                   ",
                  "   ___                  ",
                  "                  X     ",
                  "                 ___    ",
                  "           X            ",
                  "         _____          ",
                  "   X                    ",
                  "  ____                  ",
                  "                        ",
                  "          X             ",
                  "       __________       ",
                  "                        ",
                  "                 X      ",
                  "                ____    ",
                  "    X                   ",
                  "  ______                ",
                  "                        ",
                  "                        ",
                  };
        private static readonly int tileWidth = 800 / lines[0].Length;
        private static readonly int tileHeight = 480 / lines.Length;


        /// <summary>
        /// Tämä funktio laskee vihollisten lukumäärän taulukosta silmukalla
        /// </summary>
        /// <param name="vihutaulukko">Annettu taulukko josta vihut lasketaan</param>
        /// <returns></returns>
        public static int LaskeVihollistenMaara(String[] vihutaulukko)
        {
            int vihujenLkm = 0;
            for (int i = 0; i < vihutaulukko.Length; i++)
            {
                for (int j = 0; j < vihutaulukko[i].Length; j++)
                {
                    if (vihutaulukko[i][j] == 'X') vihujenLkm++;
                }
            }
            return vihujenLkm;
        }
        

        /// <summary>
        /// Begin metodi avaa alkuvalikon
        /// </summary>
        public override void Begin()
        {
            if (tarkistin)
            {
                // Kirjoita ohjelmakoodisi tähän

                string[] vaihtoehdot = { "Aloita peli", "Tehokkaimmat nukkujat", "Lopeta" };
                MultiSelectWindow alkuvalikko = new MultiSelectWindow("HIKO", vaihtoehdot);

                alkuvalikko.AddItemHandler(0, AloitaPeli);
                alkuvalikko.AddItemHandler(1, ParhaatPisteet);
                alkuvalikko.AddItemHandler(2, Exit);
                alkuvalikko.Color = Color.AshGray;
                alkuvalikko.SetButtonColor(Color.BlueGray);
                alkuvalikko.SetButtonTextColor(Color.Blue);
                Add(alkuvalikko);
            }
            tarkistin = false;
        }



        /// <summary>
        /// Tämä aliohjelma aloittaa sen varsinaisen pelin
        /// </summary>
        public void AloitaPeli()
        {

            //Level.CreateBorders();
            PhysicsObject alareuna = Level.CreateBottomBorder();
            alareuna.Restitution = 1.0;
            alareuna.IsVisible = true;

            Camera.ZoomToLevel();

            TileMap tiles = TileMap.FromStringArray(lines);

            Level.Background.Image = taustakuva;
            LuoPelaaja();
            Gravity = new Vector(0.0, -981.0);
            LuoOhjaimet();
            LuoTahtain();
            LuoAikalaskuri();
            LuoPisteLaskuri();
            tiles.SetTileMethod('X', LuoMaavihu);
            tiles.SetTileMethod('_', LuoTaso);
            tiles.Execute(tileWidth, tileHeight);
            jaljellaOlevatVihut = LaskeVihollistenMaara(lines);
            AddCollisionHandler(pelaaja, alareuna, OsuiReunaan);

            
        }


        /// <summary>
        /// Tämä aliohjelma avaa parhaat pisteet alkuvalikkoon
        /// </summary>
        void ParhaatPisteet()
        {
            tarkistin = true;
            topLista = DataStorage.TryLoad<ScoreList>(topLista, "pisteet.xml");
            HighScoreWindow topIkkuna = new HighScoreWindow("Tehokkaimmat nukkujat", topLista);
            Add(topIkkuna);
            Mouse.Listen(MouseButton.Left, ButtonState.Down, Begin, null);
        }

        
        /// <summary>
        /// Luo pelaajamodellin ja antaa sille ominaisuudet
        /// </summary>
        /// <returns></returns>
        PhysicsObject LuoPelaaja()
        {
            pelaaja = new PhysicsObject(20, 40);
            pelaaja.Shape = Shape.Circle;
            pelaaja.Color = Color.Blue;
            pelaaja.Image = LoadImage("pääjehu2");
            pelaaja.CanRotate = false;
            pelaaja.AddCollisionIgnoreGroup(1);
            AddCollisionHandler<PhysicsObject, Vihu>(pelaaja, PelaajaTormasi);
            Add(pelaaja);
            return pelaaja;
        }


        /// <summary>
        /// Kun pelaaja törmää vihuun aliohjelma
        /// </summary>
        /// <param name="tormaaja">pelaaja törmää</param>
        /// <param name="kohde">kohde johon pelaaja törmää</param>
        void PelaajaTormasi(PhysicsObject tormaaja, Vihu kohde)
        {
            PelaajaKuolee();
        }


        /// <summary>
        /// Kun pelaaja osuu kentän reunaan
        /// </summary>
        /// <param name="pelaaja"></param>
        /// <param name="kohde"></param>
        void OsuiReunaan(PhysicsObject pelaaja, PhysicsObject kohde)
        {
            PelaajaKuolee();
        }


        /// <summary>
        /// Kun pelaaja kuolee x-asian tapahtumasta, niin tämä suoritetaan
        /// </summary>
        void PelaajaKuolee()
        {
            MessageDisplay.Add("You died!");
            pelaaja.Destroy();

            this.Pause();
            topLista = DataStorage.TryLoad<ScoreList>(topLista, "pisteet.xml");

            HighScoreWindow topIkkuna = new HighScoreWindow("Tehokkaimmat nukkujat", topLista);
            Add(topIkkuna);
        }


        /// <summary>
        /// Tämä luo maavihun kentälle
        /// </summary>
        /// <param name="paikka">mihin kohtaan kentälle sen luo</param>
        /// <param name="leveys">vihun hitboxin leveys</param>
        /// <param name="korkeus">vihun hitboxin korkeus</param>
        void LuoMaavihu(Vector paikka, double leveys, double korkeus)
        {
            //PlatformCharacter maavihu = new PlatformCharacter(leveys, korkeus);
            Vihu maavihu = new Vihu(leveys, korkeus);
            maavihu.Shape = Shape.Circle;
            maavihu.Color = Color.Blue;
            maavihu.Position = paikka;
            maavihu.Tag = "maavihu";
            PlatformWandererBrain tasoaivot = new PlatformWandererBrain();
            tasoaivot.Speed = 100;
            maavihu.Brain = tasoaivot;
            maavihu.Image = LoadImage("örkki");
            
            Add(maavihu);
        }


        /// <summary>
        /// Luo näkyvän indikaattorin tähtäimestä
        /// </summary>
        /// <returns></returns>
        GameObject LuoTahtain()
        {
            tahtain = new GameObject(30.0, 30.0, Shape.Circle);
            tahtain.Image = LoadImage("tähtäinristikko");
            Add(tahtain);
            return tahtain;
        }


        /// <summary>
        /// Luo tarvittavat ohjaimet pelaajalle
        /// </summary>
        public void LuoOhjaimet()
        {
            Keyboard.Listen(Key.Left, ButtonState.Down, LiikutaPelaajaa, null, new Vector(-1000, 0));
            Keyboard.Listen(Key.Right, ButtonState.Down, LiikutaPelaajaa, null, new Vector(1000, 0));
            Keyboard.Listen(Key.Up, ButtonState.Down, LiikutaPelaajaa, null, new Vector(0, 1000));
            Keyboard.Listen(Key.Down, ButtonState.Down, LiikutaPelaajaa, null, new Vector(0, -1000));

            Keyboard.Listen(Key.Escape, ButtonState.Pressed, ConfirmExit, "Lopeta peli");

            Mouse.IsCursorVisible = true;
            Mouse.ListenMovement(0.1, KuunteleLiiketta, null);

            Mouse.Listen(MouseButton.Left, ButtonState.Pressed, Ammu, null);

        }


        /// <summary>
        /// Tämä aliohjelma liikuttaa pelaajaa tiettyyn suuntaan
        /// </summary>
        /// <param name="vektori">Minne suuntaan sitä liikutetaan</param>
        void LiikutaPelaajaa(Vector vektori)
        {
            pelaaja.Push(vektori);
        }


        /// <summary>
        /// Tässä kerrotaan missä tähtäin on ruudulla
        /// </summary>
        void KuunteleLiiketta()
        {
            tahtain.X = Mouse.PositionOnWorld.X;
            tahtain.Y = Mouse.PositionOnWorld.Y;

        }


        /// <summary>
        /// Tämä luo projektiilin jolla voi tuhota vihuja 
        /// </summary>
        void Ammu()
        {
            PhysicsObject luoti = new PhysicsObject(20, 20);
            luoti.Shape = Shape.Circle;
            luoti.Color = Color.Yellow;
            luoti.X = pelaaja.X;
            luoti.Y = pelaaja.Y;
            luoti.IgnoresGravity = true;
            luoti.IgnoresCollisionWith(pelaaja);
            luoti.AddCollisionIgnoreGroup(1);
            luoti.LifetimeLeft = TimeSpan.FromSeconds(0.3);
            luoti.Image = LoadImage("luoti");
            Add(luoti);

            //luoti.Hit(new Vector(pelaaja.RelativePosition.X - Mouse.PositionOnScreen.X, pelaaja.RelativePosition.Y - Mouse.PositionOnScreen.Y));
            Vector suunta = AnnaLuodinSuunta(pelaaja);
            Angle a = suunta.Angle;
            
            luoti.Hit(Vector.FromLengthAndAngle(200, a));
            AddCollisionHandler<PhysicsObject, Vihu>(luoti, LuotiOsuuVihuun);
        }


        /// <summary>
        /// Tässä funktio palauttaa luodin suunnan
        /// </summary>
        /// <param name="pelaaja">tarvittava positio mistä se ammutaan</param>
        /// <returns></returns>
        public Vector AnnaLuodinSuunta(PhysicsObject pelaaja)
        {
            return new Vector((pelaaja.RelativePosition.X - Mouse.PositionOnScreen.X) * -1, (pelaaja.RelativePosition.Y - Mouse.PositionOnScreen.Y) * -1);
        }


        /// <summary>
        /// Tässä kerrotaan mitä tapahtuu kun luoti osuu pelaajaan
        /// </summary>
        /// <param name="luoti">projektiili joka osuu</param>
        /// <param name="vihu">mihin osuu</param>
        private void LuotiOsuuVihuun(PhysicsObject luoti, Vihu vihu)
        {
            MessageDisplay.Add("luoti osui vihuun");
            pistelaskuri.Value++;

            if (vihu.Elamalaskuri.Value == 1) jaljellaOlevatVihut--;

            vihu.Elamalaskuri.Value--;

            if (jaljellaOlevatVihut == 0) VoitaPeli();
        }


        /// <summary>
        /// Tätä aliohjelmaa kutsutaan kun pelaaja voittaa pelin
        /// </summary>
        void VoitaPeli()
        {
            MessageDisplay.Add(Double.Parse(aikanaytto.Text).ToString());
            this.Pause();


            topLista = DataStorage.TryLoad<ScoreList>(topLista, "pisteet.xml");
            double i = Double.Parse(aikanaytto.Text);

            
            double tehokkuus = pistelaskuri.Value / i;

            HighScoreWindow topIkkuna = new HighScoreWindow("Tehokkaimmat nukkujat", "Hiivuit listalle ajalla", topLista, tehokkuus);

            topIkkuna.Closed += TallennaPisteet;
            Add(topIkkuna);
        }


        /// <summary>
        /// Tämä aliohjelma tallentaa pisteet pisteet.xml tiedostoon
        /// </summary>
        /// <param name="sender">mitkä pisteet tallennetaan</param>
        private void TallennaPisteet(Window sender)
        {
            DataStorage.Save<ScoreList>(topLista, "pisteet.xml");
        }


        /// <summary>
        /// Tämä aliohjelma luo tasanteita kenttään
        /// </summary>
        /// <param name="paikka">tason paikka koordinaatistossa</param>
        /// <param name="leveys">tason leveys</param>
        /// <param name="pituus">tason korkeus</param>
        void LuoTaso(Vector paikka, double leveys, double pituus)
        {
            Surface taso = new Surface(leveys, pituus);
            taso.Position = paikka;
            taso.Color = Color.Orange;
            Add(taso);
        }


        /// <summary>
        /// Tässä on aikalaskuri peliin
        /// </summary>
        void LuoAikalaskuri()
        {

            Jypeli.Timer aikalaskuri = new Jypeli.Timer();
            aikalaskuri.Start();

            aikanaytto = new Label();
            aikanaytto.TextColor = Color.Orange;
            aikanaytto.DecimalPlaces = 1;
            aikanaytto.BindTo(aikalaskuri.SecondCounter);
            aikanaytto.Position = new Vector(0, 300);
            Add(aikanaytto);
            
            
        }


        /// <summary>
        /// Tämä aliohjelma luo pistelaskurin
        /// </summary>
        void LuoPisteLaskuri()
        {
            pistelaskuri = new IntMeter(0);

            Label pistenaytto = new Label();
            pistenaytto.X = Screen.Left + 100;
            pistenaytto.Y = Screen.Top - 100;
            pistenaytto.TextColor = Color.Black;
            pistenaytto.Color = Color.White;

            pistenaytto.BindTo(pistelaskuri);
            Add(pistenaytto);
        }


    }
}